<?php

namespace app\models;

use Yii;

/**
 * This is the model class for table "Condicionante".
 *
 * @property string $id
 * @property string|null $titulo
 * @property string|null $descripcion
 * @property bool|null $activa
 * @property string|null $amplitud
 * @property string|null $idProducto
 * @property string|null $subproductos
 * @property string|null $idCreador
 * @property string|null $idModificador
 * @property string|null $creado
 * @property string|null $modificado
 * @property string|null $eliminado
 *
 * @property CondicionanteDestino[] $condicionanteDestino
 * @property CondicionanteFin[] $condicionanteFin
 * @property CondicionanteOrigen[] $condicionanteOrigen
 * @property CondicionanteTipo[] $condicionanteTipo
 * @property Usuario $idCreador
 * @property Estado[] $idDestino
 * @property FinMovilizacion[] $idFin
 * @property Usuario $idModificador
 * @property Estado[] $idOrigen
 * @property Producto $idProducto
 * @property TipoMovilizacion[] $idTipo
 */
class Condicionante extends ModeloBase {
    /**
     * {@inheritdoc}
     */
    public static function tableName() {
        return 'Condicionante';
    }

    /**
     * {@inheritdoc}
     */
    public function rules() {
        return [
            [['id'], 'required'],
            [['descripcion', 'subproductos'], 'string'],
            [['activa'], 'boolean'],
            [['creado', 'modificado', 'eliminado'], 'safe'],
            [['id', 'idProducto', 'idCreador', 'idModificador'], 'string', 'max' => 36],
            [['titulo'], 'string', 'max' => 100],
            [['amplitud'], 'string', 'max' => 50],
            [['id'], 'unique'],
            [['idProducto'], 'exist', 'skipOnError' => true, 'targetClass' => Producto::class, 'targetAttribute' => ['idProducto' => 'id']],
            [['idCreador'], 'exist', 'skipOnError' => true, 'targetClass' => Usuario::class, 'targetAttribute' => ['idCreador' => 'id']],
            [['idModificador'], 'exist', 'skipOnError' => true, 'targetClass' => Usuario::class, 'targetAttribute' => ['idModificador' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels() {
        return [
            'id' => 'ID',
            'titulo' => 'Titulo',
            'descripcion' => 'Descripcion',
            'activa' => 'Activa',
            'amplitud' => 'Amplitud',
            'idProducto' => 'Id Producto',
            'subproductos' => 'Subproductos',
            'idCreador' => 'Id Creador',
            'idModificador' => 'Id Modificador',
            'creado' => 'Creado',
            'modificado' => 'Modificado',
            'eliminado' => 'Eliminado',
        ];
    }

    /**
     * Gets query for [[CondicionanteDestino]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getCondicionanteDestino() {
        return $this->hasMany(CondicionanteDestino::class, ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[CondicionanteFin]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getCondicionanteFin() {
        return $this->hasMany(CondicionanteFin::class, ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[CondicionanteOrigen]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getCondicionanteOrigen() {
        return $this->hasMany(CondicionanteOrigen::class, ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[CondicionanteTipo]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getCondicionanteTipo() {
        return $this->hasMany(CondicionanteTipo::class, ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[IdCreador]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdCreador() {
        return $this->hasOne(Usuario::class, ['id' => 'idCreador']);
    }

    /**
     * Gets query for [[IdDestino]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdDestino() {
        return $this->hasMany(Estado::class, ['id' => 'idDestino'])->viaTable('CondicionanteDestino', ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[IdFin]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdFin() {
        return $this->hasMany(FinMovilizacion::class, ['id' => 'idFin'])->viaTable('CondicionanteFin', ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[IdModificador]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdModificador() {
        return $this->hasOne(Usuario::class, ['id' => 'idModificador']);
    }

    /**
     * Gets query for [[IdOrigen]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdOrigen() {
        return $this->hasMany(Estado::class, ['id' => 'idOrigen'])->viaTable('CondicionanteOrigen', ['idCondicionante' => 'id']);
    }

    /**
     * Gets query for [[IdProducto0]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdProducto() {
        return $this->hasOne(Producto::class, ['id' => 'idProducto']);
    }

    /**
     * Gets query for [[IdTipos]].
     *
     * @return \yii\db\ActiveQuery
     */
    public function getIdTipos() {
        return $this->hasMany(TipoMovilizacion::class, ['id' => 'idTipo'])->viaTable('CondicionanteTipo', ['idCondicionante' => 'id']);
    }
}
