<?php

namespace excel\controllers;

use app\models\Solicitud;
use excel\web\Controller;
use v1\models\RecursoRevision;

class SolicitudController extends Controller {

  public function actionIndex() {
    $inicio = $this->req->get("inicio", "");
    $fin = $this->req->get("fin", "");

    $query = Solicitud::find();

    if (!empty($inicio) & !empty($fin)) {
      $query->andWhere([
        "AND",
        [">=", "recepcion", $inicio],
        ["<=", "recepcion", $fin],
      ]);
    }

    $query->andWhere(['eliminado' => null]);

    if ($query->count() === 0)
      throw new \yii\web\NotFoundHttpException("No se encontrarón Solicitudes");

    $array = [];

    $titulos = [
      'Tipo de Acceso',
      'Folio de la Solicitud',
      'Sujeto Obligado',
      'Estado Actual',
      'Fecha de Recepción',
      'DT',
      'DR',
      'DA',
      'DT-RR',
      'DR-RR',
    ];

    $campos = [
      'tipoAcceso',
      'folioSolicitud',
      'sujetoObligado',
      'estadoActual',
      'fechaRecepción',
      'dt',
      'dr',
      'da',
      'dtrr',
      'drrr',
    ];

    foreach ($query->each() as $solicitud) {

      $recursosRevision = RecursoRevision::find()
        ->andWhere(['eliminado' => null, 'idSolicitud' => $solicitud->id])
        ->orderBy('creado');

      array_push($array, [
        'tipoAcceso' => $recursosRevision->count() > 0 ? 'RR' : 'S',
        'folioSolicitud' => "{$solicitud->folio} ",
        'sujetoObligado' => $solicitud->sujetoObligado->nombre,
        'estadoActual' => $solicitud->estatus->nombre,
        'fechaRecepción' => date("d/m/Y", strtotime($solicitud->recepcion)),
        'dt' => $solicitud->diasTranscurridos,
        'dr' => $solicitud->diasRestantes,
        'da' => $solicitud->diasAtencion,
        'dtrr' => $recursosRevision->one() ? $recursosRevision->one()->diasTranscurridos : "-",
        'drrr' => $recursosRevision->one() ? $recursosRevision->one()->diasRestantes : "-",
      ]);

      foreach ($recursosRevision->each() as $recurso){
        array_push($array, [
          'tipoAcceso' => "",
          'folioSolicitud' => "{$solicitud->folio} ",
          'sujetoObligado' => $solicitud->sujetoObligado->nombre,
          'estadoActual' => $solicitud->estatus->nombre,
          'fechaRecepción' => date("d/m/Y", strtotime($solicitud->recepcion)),
          'dt' => $solicitud->diasTranscurridos,
          'dr' => $solicitud->diasRestantes,
          'da' => $solicitud->diasAtencion,
          'dtrr' => $recursosRevision->one() ? $recursosRevision->one()->diasTranscurridos : "-",
          'drrr' => $recursosRevision->one() ? $recursosRevision->one()->diasRestantes : "-",
        ]);
      }

    }

    $fecha = date("d/m/Y");
    $nombre = "Solicitud " . $fecha;
    $titulo = "Solicitud " . $fecha;
    $pestania = "Solicitudes";

    return $this->Excel($titulo, $pestania, $nombre, $titulos, $campos, $array, false, $isArray = true);


  }

}
