<?php

namespace pdf\controllers;

use app\models\OficialiaPartesManifiestoDocumento;
use Exception;
use pdf\web\Controller;
use v1\controllers\EntregaInformacionController;
use v1\models\FondoLegislativo;
use v1\models\OficialiaPartesDocumento;
use v1\models\OficialiaPartesDocumentoResponsable;
use v1\models\OficialiaPartesManifiesto;
use v1\models\RecursoRevision;
use v1\models\RequisicionResponsable;
use v1\models\Solicitud;
use v1\models\Usuario;
use yii\web\HttpException;
use yii\web\NotFoundHttpException;

class AcuseSolicitudController extends Controller {

  public function actionIndex() {
  }

  public function actionFormato() {

    $idSolicitud = trim($this->req->get("solicitud", ""));
    $solicitud = null;

    if ($idSolicitud === "") {
      throw new HttpException(400, "Es necesario proporcionar una solicitud");
    }

    $solicitud = Solicitud::find()
      ->andWhere(['id' => $idSolicitud])
      ->andWhere(['eliminado' => null])
      ->with('usuario', 'media', 'sujetoObligado')
      ->one();

    $recursoRevision = RecursoRevision::find()
      ->innerJoin('Solicitud', '{{Solicitud}}.id = {{RecursoRevision}}.[[idSolicitud]]')
      ->andWhere(['idSolicitud' => $idSolicitud])
      ->andWhere(['{{Solicitud}}.eliminado' => null])
      ->andWhere(['{{RecursoRevision}}.eliminado' => null])
      ->one();

    if ($solicitud === null) {
      throw new NotFoundHttpException('No se encontró la solicitud');
    }

    $configuracion = [
      'mode' => 'utf-8',
      'format' => 'letter',
      'margin_top' => '36',
    ];

    $view = new \yii\web\View();

    $htmlPDF = $view->render("@app/modules/pdf/views/acuse-solicitud/formato", [
      "solicitud" => $solicitud,
      "recurso" => $recursoRevision
    ]);

    $header = $view->render("@app/modules/pdf/views/header/formato", [
      "titulo" => "ACUSE DE SOLICITUD DE INFORMACIÓN PÚBLICA",
    ]);


    $pdf = self::crearPDF(
      'AS-ISTAIAS-03',
      $htmlPDF,
      $configuracion,
      false,
      $header,
      '',
      ''
    );
  }

  public function actionOficio() {

    $request = \Yii::$app->request;

    $configuracion = [
      'mode' => 'utf-8',
      'format' => 'letter',
      'margin_top' => '50',
    ];

    $view = new \yii\web\View();

    $htmlPDF = $view->render("@app/modules/pdf/views/acuse-solicitud/oficio", [
      "recursoRevision" => [],
    ]);

    if ($request->get('header'))
      $header = $view->render("@app/modules/pdf/views/header/oficio-landing", [
        "titulo" => "ACUSE DE SOLICITUD DE INFORMACIÓN PÚBLICA",
      ]);
    else
      $header = $view->render("@app/modules/pdf/views/header/oficio", [
        "titulo" => "ACUSE DE SOLICITUD DE INFORMACIÓN PÚBLICA",
      ]);

    $pdf = self::crearPDF(
      'AS-ISTAIAS-03',
      $htmlPDF,
      $configuracion,
      false,
      $header,
      '',
      ''
    );
  }
}
