<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class CondicionanteController extends AuthController {

  public $modelClass = "v1\models\Condicionante";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $idProducto = trim($this->req->get("idProducto", ""));

    $query = $this->queryInicial;

    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if (!empty($buscar)) {
      $query->andWhere([
        'or',
        ['ilike', 'titulo', $buscar],
      ]);
    }

    if ($idProducto) {
      $query->andWhere(["idProducto" => $idProducto]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $fines = $this->req->getBodyParam("fines", []);
    $tipos = $this->req->getBodyParam("tipos", []);
    $origenes = $this->req->getBodyParam("origenes", []);
    $destinos = $this->req->getBodyParam("destinos", []);

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo -> uuid ();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), '');

      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->esError()
          ->mensaje("Hubo un problema al guardar el registro del Condicionante.");
      }

      // Guardar CondicionanteFines
      \v1\models\CondicionanteFin::deleteAll(['idCondicionante' => $modelo->id]);
      foreach ($fines as $fin) {
        $condicionanteFin = new \v1\models\CondicionanteFin();
        $condicionanteFin->idCondicionante = $modelo->id;
        $condicionanteFin->idFin = $fin;
        $condicionanteFin->creado = new Expression('now()');
        if (!$condicionanteFin->save()) {
          throw new \Exception("Error al guardar el CondicionanteFin");
        }
      }

      // Guardar CondicionanteTipos
      \v1\models\CondicionanteTipo::deleteAll(['idCondicionante' => $modelo->id]);
      foreach ($tipos as $tipo) {
        $condicionanteTipo = new \v1\models\CondicionanteTipo();
        $condicionanteTipo->idCondicionante = $modelo->id;
        $condicionanteTipo->idTipo = $tipo;
        $condicionanteTipo->creado = new Expression('now()');
        if (!$condicionanteTipo->save()) {
          throw new \Exception("Error al guardar el CondicionanteTipo");
        }
      }

      // Guardar CondicionanteOrigens
      \v1\models\CondicionanteOrigen::deleteAll(['idCondicionante' => $modelo->id]);
      foreach ($origenes as $origen) {
        $condicionanteOrigen = new \v1\models\CondicionanteOrigen();
        $condicionanteOrigen->idCondicionante = $modelo->id;
        $condicionanteOrigen->idOrigen = $origen;
        $condicionanteOrigen->creado = new Expression('now()');
        if (!$condicionanteOrigen->save()) {
          throw new \Exception("Error al guardar el CondicionanteOrigen");
        }
      }

      // Guardar CondicionanteDestinos
      \v1\models\CondicionanteDestino::deleteAll(['idCondicionante' => $modelo->id]);
      foreach ($destinos as $destino) {
        $condicionanteDestino = new \v1\models\CondicionanteDestino();
        $condicionanteDestino->idCondicionante = $modelo->id;
        $condicionanteDestino->idDestino = $destino;
        $condicionanteDestino->creado = new Expression('now()');
        if (!$condicionanteDestino->save()) {
          throw new \Exception("Error al guardar el CondicionanteDestino");
        }
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de Condicionante guardado con éxito.");

    } catch (\Exception $e) {
      $tran->rollBack();
      return (new Respuesta())
        ->esError()
        ->mensaje("Hubo un error en el servidor: " . $e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de Condicionante no encontrado.");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del Condicionante.");
    }

    return (new Respuesta())
      ->mensaje("Registro de Condicionante eliminado.");
  }
}
