<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;
use app\models\Municipio;
use v1\models\Nivel;
use v1\models\NivelMunicipio;

class MunicipioController extends AuthController {

  public $modelClass = "v1\models\Municipio";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $idEstado = trim($this->req->get("idEstado", ""));


    $query = $this->queryInicial->joinWith(['estado']);

    if ($idEstado !== "") {
      $query->andWhere(["Estado.id" => $idEstado]);
    }

    if ($id !== "") {
      $query->andWhere(["Municipio.id" => $id]);
    }

    if (!empty($buscar)) {
      $query->andWhere([
        'or',
        ['ilike', 'Municipio.nombre', $buscar],
        ['ilike', 'Estado.nombre', $buscar],
      ]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $niveles = $this->req->getBodyParam("niveles", []);

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), '');

      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->esError()
          ->mensaje("Hubo un problema al guardar el registro del Municipio.");
      }

      \v1\models\NivelMunicipio::deleteAll(['idMunicipio' => $modelo->id]);
      if (count($niveles) > 0) {
        foreach ($niveles as $idNivel) {
          $nivel = new NivelMunicipio();
          $nivel->uuid();
          $nivel->idNivel = $idNivel;
          $nivel->idMunicipio = $modelo->id;
          $nivel->creado = new Expression('now()');
          if (!$nivel->save()) {
            return (new Respuesta($nivel))
              ->esError()
              ->mensaje("Hubo un problema al guardar el registro del Municipio.");
          }
        }
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de Municipio guardado con éxito.");
    } catch (\Exception $e) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Hubo un error en el servidor" . $e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de Municipio no encontrado.");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del Municipio.");
    }

    return (new Respuesta())
      ->mensaje("Registro de Municipio eliminado.");
  }
}
