<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class ProductoController extends AuthController {

  public $modelClass = "v1\models\Producto";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $padre = boolval($this->req->get("padre", false));
    $idPadre = trim($this->req->get("idPadre", ""));

    $query = $this->queryInicial;

    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if ($idPadre !== "") {
      $query->andWhere(["idPadre" => $idPadre]);
    }

    if (!empty($buscar)) {
      if (is_numeric($buscar)) {
        $query->andWhere(['idSagarhpa' => $buscar]);
      } else {
        $query->andWhere(['ilike', 'nombre', $buscar]);
      }
    }

    if ($padre) {
      $query->andWhere(["IS", "idPadre", null]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $subproductos = $this->req->getBodyParam("subproductos", []);

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo -> uuid ();
        $modelo->creado = new Expression('now()');
        $modelo->idPadre = null;
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(),'');

      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->esError()
          ->mensaje("Hubo un problema al guardar el registro del Producto.");
      }

      if (count($subproductos) > 0) {
        $subproductosIds = [];
        foreach ($subproductos as $subproducto) {
          $subproductoModel = null;
          if (isset($subproducto["id"])) {
            $subproductoModel = $this->modelClass::findOne($subproducto["id"]);
          }

          if ($subproductoModel === null) {
            $subproductoModel = new $this->modelClass();
            $subproductoModel -> uuid ();
            $subproductoModel->creado = new Expression('now()');
          } else {
            $subproductoModel->modificado = new Expression('now()');
          }

          $subproductoModel->load($subproducto, '');
          $subproductoModel->idPadre = $modelo->id;

          if (!$subproductoModel->save()) {
            return (new Respuesta($subproductoModel))
              ->esError()
              ->mensaje("Hubo un problema al guardar el registro del Subproducto.");
          }

          $subproductosIds[] = $subproductoModel->id;
        }

        $query = $this->modelClass::find()
          ->andWhere(["NOT IN", "id", $subproductosIds])
          ->andWhere(["idPadre" => $modelo->id]);

        foreach ($query->each() as $subproducto) {
          $subproducto->eliminado = new Expression('now()');
          if (!$subproducto->save()) {
            return (new Respuesta($subproducto))
              ->esError()
              ->mensaje("Hubo un problema al eliminar el registro del Subproducto.");
          }
        }
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de Producto guardado con éxito.");

    } catch (\Exception $e) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Hubo un error en el servidor" . $e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de Producto no encontrado.");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del Producto.");
    }

    return (new Respuesta())
      ->mensaje("Registro de Producto eliminado.");
  }
}
