<?php

namespace v1\controllers;

use common\data\Respuesta;
use common\rest\AuthController;
use yii\db\Expression;

class TipoMovilizacionController extends AuthController {

  public $modelClass = "v1\models\TipoMovilizacion";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));

    $query = $this->queryInicial;

    if ($id !== "") {
      $query->andWhere(["id" => $id]);
    }

    if (!empty($buscar)) {
      if (is_numeric($buscar)) {
        $query->andWhere(['idSagarhpa' => $buscar]);
      } else {
        $query->andWhere(['ilike', 'nombre', $buscar]);
      }
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));

    $modelo = null;
    if ($id !== "") {
      $modelo = $this->modelClass::findOne($id);
    }

    $tran = \Yii::$app->getDb()->beginTransaction();
    try {
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo -> uuid ();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(),'');

      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->esError()
          ->mensaje("Hubo un problema al guardar el registro de Tipo de Movilizacion.");
      }

      $tran->commit();
      $modelo->refresh();

      return (new Respuesta($modelo))
        ->mensaje("Registro de Tipo de Movilizacion guardado con éxito.");

    } catch (\Exception $e) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Hubo un error en el servidor" . $e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", null));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Registro de Tipo de Movilizacion no encontrado.");
    }

    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar el registro del Tipo de Movilizacion.");
    }

    return (new Respuesta())
      ->mensaje("Registro de Tipo de Movilizacion eliminado.");
  }
}
